@@ -4,7 +4,6 @@
 
 import os.path
 
-import sys
 import stat  # file permissions
 import argparse
 from argparse import RawTextHelpFormatter
@@ -45,7 +44,7 @@ def parse_arguments():
     parser.add_argument('-p', '--platform', action='store', default='default', help='platform cmake cache module to load [default]')
     parser.add_argument('--list-toolchains', action='store_true', help='list all currently available toolchain modules [false]')
     parser.add_argument('--list-platforms', action='store_true', help='list all currently available platform modules [false]')
-    parser.add_argument('--track', action='store', default=None, help='retrieve rosinstalls relevant to this track [groovy|hydro][groovy]')
+    parser.add_argument('--track', action='store', default=None, help='default the underlays to this track if catkin is not found [groovy|hydro][groovy]')
     args = parser.parse_args()
     return args
 
@@ -91,7 +90,7 @@ def instantiate_template(filename, name, cwd):
         f.close()
 
 
-def fill_in_config_cmake(template, config_build_type, config_install_prefix, config_underlays, config_override_file):
+def fill_in_config_cmake(template, config_build_type, config_devel, config_install_prefix, config_underlays, config_override_file):
     return template % locals()
 
 
@@ -104,7 +103,8 @@ def instantiate_config_cmake(platform_content, build_path, config_build_type, co
     '''
     template_dir = os.path.join(os.path.dirname(__file__), 'cmake')
     template = read_template(os.path.join(template_dir, "config.cmake"))
-    contents = fill_in_config_cmake(template, config_build_type, config_install_prefix, config_underlays, common.override_filename())
+    config_devel = os.path.join(build_path, 'devel')
+    contents = fill_in_config_cmake(template, config_build_type, config_devel, config_install_prefix, config_underlays, common.override_filename())
     config_cmake_file = os.path.join(build_path, "config.cmake")
     try:
         f = open(config_cmake_file, 'w')
@@ -137,35 +137,58 @@ def print_build_details(build_dir, source_dir, install_prefix, build_type, under
     console.pretty_println("**********************************************************************************\n", console.bold)
 
 
+def get_toolchains_or_platforms(base_path):
+    '''
+      Does a look up in the path for either toolchain or platform files.
+      @param base_path : start of the search (ends in either toolchains or platforms).
+    '''
+    d = {}
+    for (dirpath, unused_dirname, filenames) in os.walk(base_path):
+        if dirpath != base_path:  # skip the root, only find files in subfolders
+            family = os.path.basename(dirpath)
+            d[family] = []
+            for filename in filenames:
+                d[family].append(os.path.splitext(filename)[0])  # leave off the .cmake extension
+    return d
+
+
 def list_toolchains():
     console.pretty_println("\n******************************** Toolchain List **********************************", console.bold)
-    for (unused_dirpath, unused_dirname, filenames) in os.walk(os.path.join(os.path.dirname(__file__), 'toolchains')):
-        for filename in filenames:
-            console.pretty_println(" -- %s" % os.path.splitext(os.path.basename(filename))[0], console.cyan)
+    toolchains = get_toolchains_or_platforms(os.path.join(os.path.dirname(__file__), 'toolchains'))
+    console.pretty_println("Official:", console.bold)
+    for family in toolchains:
+        for platform in toolchains[family]:
+            console.pretty_print(" -- %s/" % family, console.cyan)
+            console.pretty_println("%s" % platform, console.yellow)
+    console.pretty_println("Custom:", console.bold)
+    toolchains = get_toolchains_or_platforms(os.path.join(settings.yujin_tools_home(), 'toolchains'))
+    if toolchains:
+        for family in toolchains:
+            for platform in toolchains[family]:
+                console.pretty_print(" -- %s/" % family, console.cyan)
+                console.pretty_println("%s" % platform, console.yellow)
+    else:
+        console.pretty_println(" -- ", console.cyan)
     console.pretty_println("**********************************************************************************\n", console.bold)
 
 
-def get_platforms():
-    platforms = {}
-    for (dirpath, unused_dirname, filenames) in os.walk(os.path.join(os.path.dirname(__file__), 'platforms')):
-        family = os.path.basename(dirpath)
-        if family != 'platforms':
-            platforms[family] = []
-            # we are in a subdirectory, i.e. a family, good!
-            for filename in filenames:
-                platforms[family].append(os.path.splitext(filename)[0])  # leave off the .cmake extension
-    return platforms
-
-
 def list_platforms():
     console.pretty_println("\n********************************* Platform List **********************************", console.bold)
-    platforms = get_platforms()
+    platforms = get_toolchains_or_platforms(os.path.join(os.path.dirname(__file__), 'platforms'))
     console.pretty_println("Official:", console.bold)
     for family in platforms:
         for platform in platforms[family]:
             console.pretty_print(" -- %s/" % family, console.cyan)
             console.pretty_println("%s" % platform, console.yellow)
     console.pretty_println("Custom:", console.bold)
+    platforms = get_toolchains_or_platforms(os.path.join(settings.yujin_tools_home(), 'platforms'))
+    if platforms:
+        for family in platforms:
+            for platform in platforms[family]:
+                console.pretty_print(" -- %s/" % family, console.cyan)
+                console.pretty_println("%s" % platform, console.yellow)
+    else:
+        console.pretty_println(" -- ", console.cyan)
     console.pretty_println("**********************************************************************************\n", console.bold)
 
 
@@ -232,7 +255,7 @@ def init_configured_build(build_dir_="./", source_dir_="./src", underlays_="/opt
     if os.path.isfile(os.path.join(source_dir, 'catkin', 'cmake', 'toplevel.cmake')):
         catkin_toplevel = os.path.join(source_dir, 'catkin', 'cmake', 'toplevel.cmake')
     else:
-        catkin_toplevel, unused_catkin_python_path = common.find_catkin(underlays_list)
+        catkin_toplevel, unused_catkin_python_path, unused_catkin_cmake_path = common.find_catkin(underlays_list)
 
     ##########################
     # Add toplevel if exists
@@ -271,26 +294,51 @@ def init_configured_build(build_dir_="./", source_dir_="./src", underlays_="/opt
     ##########################
     if not toolchain_ == "":
         toolchains_dir = os.path.join(os.path.dirname(__file__), 'toolchains')
-        if os.path.isfile(os.path.join(toolchains_dir, toolchain_ + ".cmake")):
-            shutil.copy(os.path.join(toolchains_dir, toolchain_ + ".cmake"), os.path.join(build_dir, "toolchain.cmake"))
+        custom_toolchains_dir = os.path.join(settings.yujin_tools_home(), 'toolchains')
+        tmp_list = toolchain_.split('/')
+        if len(tmp_list) != 2:
+            raise RuntimeError("Toolchain specification invalid, must be <family>/<tuple> [%s]" % toolchain_)
+        family = tmp_list[0]
+        toolchain_tuple = tmp_list[1]
+        toolchains = get_toolchains_or_platforms(toolchains_dir)
+        custom_toolchains = get_toolchains_or_platforms(custom_toolchains_dir)
+        if not family in toolchains and not family in custom_toolchains:
+            raise RuntimeError("No toolchains available for family %s" % family)
+        if family in toolchains and toolchain_tuple in toolchains[family]:
+            toolchain_file = os.path.join(toolchains_dir, family, toolchain_tuple + ".cmake")
+        elif family in custom_toolchains and toolchain_tuple in custom_toolchains[family]:
+            toolchain_file = os.path.join(custom_toolchains_dir, family, toolchain_tuple + ".cmake")
+        else:
+            raise RuntimeError("Platform %s for family %s not available." % (family, toolchain_tuple))
+        if os.path.isfile(toolchain_file):
+            shutil.copy(toolchain_file, os.path.join(build_dir, "toolchain.cmake"))
+        else:
+            raise RuntimeError("Toolchain module not available [%s]" % toolchain_file)
 
     ##########################
     # Platform
     ##########################
     platform_content = ""
+    platforms_dir = os.path.join(os.path.dirname(__file__), 'platforms')
+    custom_platforms_dir = os.path.join(settings.yujin_tools_home(), 'platforms')
     if not platform_ == "default":
         tmp_list = platform_.split('/')
         if len(tmp_list) != 2:
             raise RuntimeError("Platform specification invalid, must be <family>/<platform type> [%s]" % platform_)
         family = tmp_list[0]
         platform = tmp_list[1]
-        platforms = get_platforms()
-        try:
-            if not platform in platforms[family]:
-                raise RuntimeError("Platform %s for family %s not available." % (family, platform))
-        except KeyError:
+        platforms = get_toolchains_or_platforms(platforms_dir)
+        custom_platforms = get_toolchains_or_platforms(custom_platforms_dir)
+        if not family in platforms and not family in custom_platforms:
             raise RuntimeError("No platforms available for family %s" % family)
-    platform_file = os.path.join(os.path.dirname(__file__), 'platforms', platform_ + ".cmake")
+        if family in platforms and platform in platforms[family]:
+            platform_file = os.path.join(platforms_dir, family, platform + ".cmake")
+        elif family in custom_platforms and platform in custom_platforms[family]:
+            platform_file = os.path.join(custom_platforms_dir, family, platform + ".cmake")
+        else:
+            raise RuntimeError("Platform %s for family %s not available." % (family, platform))
+    else:
+        platform_file = os.path.join(platforms_dir, 'default.cmake')
     if os.path.isfile(platform_file):
         f = open(platform_file, 'r')
         try:
